#include "GraphIn.h"

/************************************/
static int errHalt(const char *format, char * msg, int exitCode)
/* Puts msg formatted by format to stderr and exist with exitcode */
{
  fprintf(stderr, format, msg);
  exit(exitCode);
}
/************************************/
int findNode( int id )
/* returns index in adjacency matrix
  of a node given by its external id
  Input:
    id - external id
  Returns:
    index in Adj or -1 if not found
*/
{
  int i;
  for ( i = 0; i < MAXNODES && nodeNames[i] >= 0; i++)
    if ( id == nodeNames[i] ) return i;
  return -1;
}
/************************************/
void readData()
/* Reads data for a graph (problem 7.1)
    taggged lines are:
            V_nodes n1 n2 n3 ...
            V_arcs (n1 n2) (n3 n4) (n5 n6) ...
            V'_arcs n1 n2 n3 ...
       where n1, n2, ... are positive integers.
       V_nodes must precede V_arcs line.
       First three chars must begin in first column.
       All other lines are ignored.
   Assumes:
    external global fpIn text stream opened for reading
    external global nodeNames array
    external global nodesSubG array
    external global adjacency matrix Adj
*/
{
  static char buf[8192];
  int i, n, m, head, tail, haveNodes = 0;
  
  /* initialize node names for graph and subgraph to
      no node value
  */
  for ( i = 0; i < MAXNODES; i++ )
  {
    nodeNames[i] = NONODE;
    nodesSubG[i] = NONODE;
    for ( n = 0; n < MAXNODES; n++ )
      Adj[MAXNODES][MAXNODES] = 0;
  }
  while (fgets(buf, sizeof(buf), fpIn))
  {
    if (toupper(buf[0]) == 'V')
    { /* possible valid statement beginning */
      if (buf[1] == '_')
      { /* V_nodes or V_arcs */
        n = 3;
        if (tolower(buf[2]) == 'n')
        { /* V_nodes */
          /* skip to n1, n2,.. area */
          while ( !isspace(buf[n]) &&
                  !isdigit(buf[n]) ) n++;
          i=0;
          while ( 1 == sscanf(&buf[n], "%d%n", &nodeNames[i], &m ))
          { /* read current node id */
            n += m;
            if (i < MAXNODES - 1) i++;
            else
            {
              nodeNames[MAXNODES-1] = NONODE;
              break;
            }
          }
          if ( i ) haveNodes = 1;
        }
        else
        if (tolower(buf[2]) == 'a')
        { /* V_arcs */
          if ( !haveNodes ) errHalt("No node identifiers read yet:\n%s", buf, 2);
          while ( buf[n] != '(' ) n++; /* skip to first pair */
          while ( 2 == sscanf(&buf[n], "(%d%d)%n", &tail, &head, &m ))
          { /* read current node id */
            n += m;
            head = findNode( head );
            tail = findNode( tail );
            if (head >= 0 && tail >= 0 )
            {
              Adj[ tail ][ head ] = 1;
            }
            else
            {
              fprintf(stderr, "In line:\n%sInvalid arc in pair preceding position %d\n", buf, n);
            }
            while ( buf[n] != '(' && buf[n] !='\n' ) n++; /* skip to next pair */
            if (buf[n] =='\n') break;
          }
        }
        else
        {
          errHalt("Unknown tag in line:\n", buf, 1);
        }
      }
    }
    else if ( buf[0] == '.' ) break;
  }
}
